import numpy as np
import plotly.graph_objects as go
import feffery_antd_components as fac

from dash_extensions.enrich import DashProxy, Serverside, ServersideOutputTransform
from plotly_resampler import FigureResampler
from plotly_resampler.aggregation import MinMaxLTTB
from dash import dcc, Dash, html, Input, Output, State, no_update, callback_context

# app = Dash(__name__, server=False)
app = DashProxy(__name__, transforms=[ServersideOutputTransform()], server=False)

# server = app.server

app.layout = html.Div(
    [
        fac.AntdButton("render", id="figure-btn"),
        fac.AntdSpin(dcc.Graph(id="demo-figure")),
        fac.AntdSpin(dcc.Store(id="store")),
    ]
)

# Data that will be used for the plotly-resampler figures
x = np.arange(100000)
sin = (3 + np.sin(x / 200) + np.random.randn(len(x)) / 10) * x / 1_000


@app.callback(
    [Output("demo-figure", "figure"), Output("store", "data")],
    Input("figure-btn", "nClicks"),
    prevent_initial_call=True,
)
def generate_figure(n_clicks):
    ctx = callback_context
    if len(ctx.triggered) and "figure-btn" in ctx.triggered[0]["prop_id"]:
        fig: FigureResampler = FigureResampler(
            default_downsampler=MinMaxLTTB(parallel=True)
        )
        fig.add_trace(
            go.Scattergl(
                x=x, y=sin, name="demo", mode="lines+markers", showlegend=True
            ),
            max_n_samples=10_000,
        )
        fig.update_layout(
            legend=dict(orientation="h", yanchor="bottom", y=1.02, xanchor="right", x=1)
        )
        return fig, Serverside(fig)
    return no_update


# The plotly-resampler callback to update the graph after a relayout event (= zoom/pan)
# As we use the figure again as output, we need to set: allow_duplicate=True
@app.callback(
    Output("demo-figure", "figure", allow_duplicate=True),
    Input("demo-figure", "relayoutData"),
    State("store", "data"),  # The server side cached FigureResampler per session
    prevent_initial_call=True,
    memoize=True,
)
def update_fig(relayoutdata: dict, fig: FigureResampler):
    if fig is None:
        return no_update
    return fig.construct_update_data_patch(relayoutdata)
